#include "Nd2MemoryTest.h"

//Post 2 receives on local QP and flush the QP. 
//Verify 1 CQ notification is generated
//Verify the next 2 receive request completed with cancelled status due to QP flush

void Nd2ReceiveFlushQPServer::RunTest(
    _In_ const struct sockaddr_in& v4Src,
    _In_ DWORD queueDepth,
    _In_ DWORD nSge
    )
{
	//prep
    Nd2TestBase::Init( v4Src);
	Nd2TestBase::CreateMR();
	Nd2TestBase::RegisterDataBuffer( x_HdrLen + x_MaxXfer+1,ND_MR_FLAG_ALLOW_LOCAL_WRITE,ND_SUCCESS,"Register memory with local read access failed");
	Nd2TestBase::CreateCQ(nSge);
	Nd2TestBase::CreateConnector();
	Nd2TestBase::CreateQueuePair(queueDepth,nSge);
	Nd2TestServerBase::CreateListen();
	Nd2TestServerBase::Listen(v4Src);
	
    // Pre-post 2 receive requests.
	Nd2TestBase::PostReceive(NULL,0);
	Nd2TestBase::PostReceive(NULL,0);

	//flush QP
	Nd2TestBase::FlushQP();
	
	//Expect 1 generated notification due to flush
	ND2_RESULT pResult[2];
	HRESULT hr = m_pCq->Notify( ND_CQ_NOTIFY_ANY, &m_Ov );
	if( hr == ND_PENDING )
	{
		hr = m_pCq->GetOverlappedResult( &m_Ov, TRUE );
	}

	//expecting two results,both being ND_CANCELED
	int numResults = m_pCq->GetResults(pResult,2);
	if(numResults!=2){
		printf("Expecting 2 results, but only getting %d",numResults);
		LogErrorExit("Error in CompletionQueue::GetResults",__LINE__);
	}
	else
	{
		LogIfErrorExit(pResult[0].Status,ND_CANCELED,"Expecting receive cancelled due to QP flush",__LINE__);
		LogIfErrorExit(pResult[1].Status,ND_CANCELED,"Expecting receive cancelled due to QP flush",__LINE__);
	}

}





#pragma warning( disable : 4100 )

void Nd2ReceiveFlushQPClient::RunTest(
    _In_ const struct sockaddr_in& v4Src,
    _In_ const struct sockaddr_in& v4Dst,
    _In_ DWORD queueDepth,
    _In_ DWORD nSge
    )
{
    //do nothing on the client
}

